<?php
namespace Drupal\entity_link_display\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'entity_link_display' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_link_display",
 *   label = @Translation("Display Link"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class EntityLinkDisplayFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'link_text' => 'View content',
      'link_class' => '',
      'link_rel' => [],
      'link_target' => '_self',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [];

    $elements['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link text'),
      '#default_value' => $this->getSetting('link_text'),
      '#description' => $this->t('Text to display for the link.'),
    ];

    $elements['link_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS class'),
      '#default_value' => $this->getSetting('link_class'),
      '#description' => $this->t('Optional CSS classes to apply to the link.'),
    ];

    $elements['link_rel'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Rel attributes'),
      '#options' => [
        'nofollow' => $this->t('nofollow'),
        'noopener' => $this->t('noopener'),
        'noreferrer' => $this->t('noreferrer'),
        'external' => $this->t('external'),
      ],
      '#default_value' => $this->getSetting('link_rel'),
      '#description' => $this->t('Select one or more rel attributes to add.'),
    ];

    $elements['link_target'] = [
      '#type' => 'select',
      '#title' => $this->t('Target'),
      '#options' => [
        '_self' => $this->t('Same tab (_self)'),
        '_blank' => $this->t('New tab (_blank)'),
        '_parent' => $this->t('Parent frame (_parent)'),
        '_top' => $this->t('Top frame (_top)'),
      ],
      '#empty_option' => $this->t('- None -'),
      '#default_value' => $this->getSetting('link_target'),
      '#description' => $this->t('How the link should open.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary[] = $this->t('Link text: @text', ['@text' => $this->getSetting('link_text')]);
    $summary[] = $this->t('CSS class: @class', ['@class' => $this->getSetting('link_class')]);

    $rel = array_filter($this->getSetting('link_rel'));
    $summary[] = $this->t('Rel: @rel', ['@rel' => implode(', ', $rel)]);

    $summary[] = $this->t('Target: @target', ['@target' => $this->getSetting('link_target')]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $entity = $items->getEntity();

    if ($entity && $entity->hasLinkTemplate('canonical')) {
      $url = $entity->toUrl();

      $link_text = $this->getSetting('link_text') ?: $entity->label();
      $link_class = $this->getSetting('link_class');
      $link_rel = array_filter($this->getSetting('link_rel'));
      $link_target = $this->getSetting('link_target');

      $attributes = [];
      if (!empty($link_class)) {
        $attributes['class'] = array_filter(explode(' ', $link_class));
      }
      if (!empty($link_rel)) {
        $attributes['rel'] = $link_rel;
      }
      if (!empty($link_target)) {
        $attributes['target'] = $link_target;
      }

      $elements[] = [
        '#type' => 'link',
        '#title' => $link_text,
        '#url' => $url,
        '#attributes' => $attributes,
      ];
    }

    return $elements;
  }
}
